extern char *progname;

void *emalloc(size_t size, char *filename);
void *xmalloc(size_t size, char *filename);
void *erealloc(void *buf, size_t size, char *filename);
void *xrealloc(void *buf, size_t size, char *filename);
void error(char *filename);
void die(char *filename);

struct stream {
	size_t (*read)(struct stream *s, char *buf, size_t size); /* returns the
		number of bytes read */
	int (*write)(struct stream *s, char *buf, size_t size); /* returns -1 on
		error; 0 on success */
	void (*close)(struct stream *s);
	int (*error)(struct stream *s);
	char *filename;
};

struct stream *file_stream_new(FILE *f, char *filename);
struct stream *file_stream_open(char *filename, char *mode);
struct stream *gz_stream_open(char *filename, char *mode);
struct stream *gz_compress_dopen(int fd, char *filename, int level);
struct stream *bz2_stream_open(char *filename, char *mode);
struct stream *bz2_compress_dopen(int fd, char *filename, int level);
struct stream *xz_stream_open(char *filename, char *mode);
struct stream *xz_compress_dopen(int fd, char *filename, int level);

size_t x_stream_read(struct stream *s, char *buf, size_t size);
void x_stream_write(struct stream *s, char *buf, size_t size);
void x_stream_seek(struct stream *s, long offset);

size_t copy_all(struct stream *in, struct stream *out);
size_t x_copy_all(struct stream *in, struct stream *out);
size_t copy_bytes(struct stream *in, struct stream *out, size_t size);
void fill_bytes(struct stream *out, size_t size);

char *construct_child_path(char *parent, char *child);
int path_is_safe(char *path);
