struct extractor {
	/* 1: success; 0: directory missing; -1: error */
	int (*write)(struct extractor *e, struct stream *in, char *filename,
		mode_t mode, time_t modtime, size_t size, int track, int reopen);
	/* 1: success; 0: directory missing; -1: failure */
	int (*symlink)(struct extractor *e, char *target, char *filename,
		int track);
	/* 1: success; 0: directory missing; -1: failure */
	int (*link)(struct extractor *e, char *target, char *filename, int track);
	/* 1: success; 0: directory missing; -1: failure */
	int (*mknod)(struct extractor *e, char *filename, mode_t mode, dev_t dev,
		int track);
	/* 1: success; 0: directory missing; -1: failure */
	int (*mkdir)(struct extractor *e, char *filename, mode_t mode, int track);
};

struct dep {
	struct dep *next;
	char *name;
	char *min_version;
	char *max_version;
};

struct stringlist {
	struct stringlist *next;
	char *s;
};

struct path_info {
	struct path_info *next;
	char *name;
	dev_t device;
	ino_t inode;
	int link;
};

int checksum(char *block, int sgn);
int checksum_is_valid(char *buf, char *filename);

int path_is_safe(char *path);

/* -1: error, 0: success, 1: success for directory */
int prepare_single_path_info(struct path_info **pi, char *name, int linkfirst);

int prepare_path(struct path_info **pi, char *parent, char *path,
	int linkfirst);

/* 1: yes, 0: no, -1: error */
int paths_equal(char *s, char *t);

/* 0: success, -1: failure */
int output_path(struct stream *out, struct path_info *pi,
	struct path_info *opi, struct stringlist *untracked, uid_t uid,
	char *uname, gid_t gid, char *gname);

void output_footer(struct stream *out);

/* -1: error, 0: containing folder missing, 1: nothing done, 2: success */
int ex_write(struct stream *in, char *filename, mode_t mode, time_t modtime,
	size_t size, int flags, int reopen, char *tgtdir);
/* -1: error, 0: containing folder missing, 1: nothing done, 2: success */
int ex_symlink(char *target, char *filename, int flags, char *tgtdir);
/* -1: error, 0: containing folder missing, 1: nothing done, 2: success */
int ex_link(char *target, char *filename, int flags, char *tgtdir);
/* -1: error, 0: containing folder missing, 1: nothing done, 2: success */
int ex_mknod(char *filename, mode_t mode, dev_t dev, int flags, char *tgtdir);
/* -1: error, 0: containing folder missing, 1: nothing done, 2: success */
int ex_mkdir(char *filename, mode_t mode, int flags, char *tgtdir);

/* 0: success, -1: failure */
int guarantee_parent_dir(struct extractor *e, char *ffn, char *filename,
	mode_t mode, int track);

/* 0: success, -1: failure */
int extractor_write(struct extractor *e, struct stream *in, char *tgtdir,
	char *filename, mode_t mode, time_t modtime, size_t size, int track,
	int reopen);

/* 0: success, -1: failure */
int extractor_link(struct extractor *e, char *tgtdir, char *filename,
	char *target, int track);

/* 0: success, -1: failure */
int extractor_symlink(struct extractor *e, char *tgtdir, char *filename,
	char *target, int track);

/* 0: success, -1: failure */
int extractor_mknod(struct extractor *e, char *tgtdir, char *filename,
	dev_t dev, mode_t mode, int track);

/* 0: success, -1: failure */
int guarantee_dir(struct extractor *e, char *ffn, char *filename, mode_t mode,
	int track);

/* 1: file processed successfully, 0: no more files,
	-1: file processed with errors */
int extract_file(struct extractor *e, char *tgtdir, struct stream *in,
	int trackable);

int write_kvp(FILE *f, char *key, char *value);
int read_kvp(FILE *f, char *filename, char **key, char **value);

#define EXTRACT_KEEP 1
#define EXTRACT_PERMISSION 2
#define EXTRACT_NOISYKEEP 4

struct extractor *new_tar_extractor(int flags);
struct extractor *new_list_extractor();

void target_emsg(char *target, char *filename, char *emsg);
void target_error(char *target, char *filename);
void target_die(char *target, char *filename);
